﻿Type=Class
Version=3.8
B4A=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Dim Manager As UsbManager
	Dim Interface As UsbInterface
	Dim Device As UsbDevice
	Dim Connection As UsbDeviceConnection
	Dim InEndPoint As UsbEndpoint
	Dim OutEndPoint As UsbEndpoint
	Dim USBAvailable As Boolean
	Dim MidiThruMap As Map
	Dim MidiThru As Boolean
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	Manager.Initialize
	MidiThruMap.Initialize
End Sub
'Get attached USB devices
Sub GetDevice As Boolean
	Dim usbdevices() As UsbDevice
	usbdevices = Manager.GetDevices

	'Iterate over devices and find the correct one
	For i = 0 To usbdevices.Length - 1
		Dim ud As UsbDevice
		ud = usbdevices(i)
		'Check if the interface is the one we want
		If GetInterface(ud) Then Exit
	Next
	'If not initialized (didn't find a suitable device)
	If Not(Device.IsInitialized) Then 
		Log("USB Midi Device not found.")
		ToastMessageShow("USB Midi Device not found.",False)
		Return False
	End If
	Return True
End Sub
Sub GetInterface(ud As UsbDevice) As Boolean
	Dim Found As Boolean
	'Iterate over interfaces
	For a = 0 To ud.InterfaceCount - 1
	 Dim inter As UsbInterface
	 inter = ud.GetInterface(a)
	 'Is it a midi USB device?
	 If inter.InterfaceClass = 1 AND inter.InterfaceSubclass = 3 Then
	 	Found = True
	    'Found our device and interface
	    Device = ud
	    Interface = inter
	    'Find correct endpoints
	    For b = 0 To Interface.EndpointCount - 1
	       Dim endpoint As UsbEndpoint
	       endpoint = Interface.GetEndpoint(b)
	       If endpoint.Type = Manager.USB_ENDPOINT_XFER_BULK Then
	          If endpoint.Direction = Manager.USB_DIR_IN Then 
	             InEndPoint = endpoint
	          Else If endpoint.Direction = Manager.USB_DIR_OUT Then
	             OutEndPoint = endpoint
	          End If
	       End If
	    Next
	 End If
	Next
	If Found Then
		'Check we have permission to use the device
		If Manager.HasPermission(Device) = False Then
			Manager.RequestPermission(Device)
			If Manager.HasPermission(Device) = False Then 
				ToastMessageShow("Permission not granted",False)
				Return False
			End If
		End If
		'Get the connection
		Connection = Manager.OpenDevice(Device, Interface, True)
		If Connection.IsInitialized Then
			USBAvailable = True
			Return True
		Else
			Log("Connection Failed")
			USBAvailable = False
			Return False
		End If
	End If
	'Return result
	Return Found
End Sub
'Turn on XG mode for yamaha devices so drums play on channel 10
Sub TurnXGOn
	Dim MidiMsg() As Byte = Array As Byte(0xF0,0x43,0x10,0x4C,0x00,0x00,0x7E,0x00,0xF7)
	SendSysex(MidiMsg)
End Sub
'Make sure longer messages are sent in 4 byte packets with correct packed headers
Sub SendSysex(SysexMsg() As Byte)
	Dim i As Int
'	Dim SLB As SLByteArrayBuffer
'	SLB.Initialize(10)
	Do While i < SysexMsg.Length - 2
		Dim MidiMsg(4) As Byte = Array As Byte(0x04,0,0,0)	
		MidiMsg(1) = SysexMsg(i)
		MidiMsg(2) = SysexMsg(i+1)
		MidiMsg(3) = SysexMsg(i+2)
		i=i+3
		If i = SysexMsg.Length Then MidiMsg(0) = 0x07
		Connection.BulkTransfer(OutEndPoint,MidiMsg,MidiMsg.Length,0)
'		SLB.AppendByte(MidiMsg,0,MidiMsg.Length)
	Loop
	
	If i < SysexMsg.Length Then
		'Deal with what's left
		Select SysexMsg.Length - i
			Case 2
				Dim MidiMsg(4) As Byte = Array As Byte(0x06,0,0,0)	
				MidiMsg(1) = SysexMsg(i)
				MidiMsg(2) = SysexMsg(i+1)
			Case 1
				Dim MidiMsg(4) As Byte = Array As Byte(0x05,0,0,0)	
				MidiMsg(1) = SysexMsg(i)
		End Select
'		SLB.AppendByte(MidiMsg,0,MidiMsg.Length)
		Connection.BulkTransfer(OutEndPoint,MidiMsg,MidiMsg.Length,0)
	End If
'	Utils.LogByteArray(SLB.ToByteArray)
End Sub
